@@ -6,7 +6,7 @@
  * \brief   Command-line and config file processing for fwknop client.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -124,6 +124,7 @@ enum
     FWKNOP_CLI_ARG_NAT_ACCESS,
     FWKNOP_CLI_ARG_HTTP_USER_AGENT,
     FWKNOP_CLI_ARG_RESOLVE_URL,
+    FWKNOP_CLI_ARG_SERVER_RESOLVE_IPV4,
     FWKNOP_CLI_ARG_NAT_LOCAL,
     FWKNOP_CLI_ARG_NAT_RAND_PORT,
     FWKNOP_CLI_ARG_NAT_PORT,
@@ -172,6 +173,7 @@ static fko_var_t fko_var_array[FWKNOP_CLI_LAST_ARG] =
     { "NAT_ACCESS",            FWKNOP_CLI_ARG_NAT_ACCESS            },
     { "HTTP_USER_AGENT",       FWKNOP_CLI_ARG_HTTP_USER_AGENT       },
     { "RESOLVE_URL",           FWKNOP_CLI_ARG_RESOLVE_URL           },
+    { "SERVER_RESOLVE_IPV4",   FWKNOP_CLI_ARG_SERVER_RESOLVE_IPV4   },
     { "NAT_LOCAL",             FWKNOP_CLI_ARG_NAT_LOCAL             },
     { "NAT_RAND_PORT",         FWKNOP_CLI_ARG_NAT_RAND_PORT         },
     { "NAT_PORT",              FWKNOP_CLI_ARG_NAT_PORT              },
@@ -199,7 +201,7 @@ static int critical_var_array[] =
 };
 
 /**
- * @brief Generate Rijndael + HMAC keys from /dev/random (base64 encoded) and exit.
+ * @brief Generate Rijndael + HMAC keys from /dev/urandom (base64 encoded).
  *
  * @param options FKO command line option structure
  */
@@ -289,7 +291,8 @@ add_var_to_bitmask(short var_pos, fko_var_bitmask_t *bm)
 
     /* The index on the uint32_t bitmask is invalid */
     else
-        log_msg(LOG_VERBOSITY_WARNING, "add_var_to_bitmask() : Bad variable position %u", var_pos);
+        log_msg(LOG_VERBOSITY_WARNING,
+                "add_var_to_bitmask() : Bad variable position %u", var_pos);
 }
 
 /**
@@ -315,7 +318,8 @@ remove_var_from_bitmask(short var_pos, fko_var_bitmask_t *bm)
 
     /* The index on the uint32_t bitmask is invalid */
     else
-        log_msg(LOG_VERBOSITY_WARNING, "remove_from_bitmask() : Bad variable position %u", var_pos);
+        log_msg(LOG_VERBOSITY_WARNING,
+                "remove_from_bitmask() : Bad variable position %u", var_pos);
 }
 
 /**
@@ -1195,6 +1199,14 @@ parse_rc_param(fko_cli_options_t *options, const char *var_name, char * val)
         }
         strlcpy(options->resolve_url, val, tmpint);
     }
+    /* Resolve the SPA server (via DNS) - accept IPv4 addresses only ? */
+    else if (var->pos == FWKNOP_CLI_ARG_SERVER_RESOLVE_IPV4)
+    {
+        if (is_yes_str(val))
+        {
+            options->spa_server_resolve_ipv4 = 1;
+        }
+    }
     /* wget command */
     else if (var->pos == FWKNOP_CLI_ARG_WGET_CMD)
     {
@@ -1417,6 +1429,9 @@ add_single_var_to_rc(FILE* fhandle, short var_pos, fko_cli_options_t *options)
                 strlcpy(val, options->resolve_url, sizeof(val));
             else;
             break;
+        case FWKNOP_CLI_ARG_SERVER_RESOLVE_IPV4:
+            bool_to_yesno(options->spa_server_resolve_ipv4, val, sizeof(val));
+            break;
         case FWKNOP_CLI_ARG_NAT_LOCAL :
             bool_to_yesno(options->nat_local, val, sizeof(val));
             break;
@@ -1449,12 +1464,15 @@ add_single_var_to_rc(FILE* fhandle, short var_pos, fko_cli_options_t *options)
             bool_to_yesno(options->no_save_args, val, sizeof(val));
             break;
         default:
-            log_msg(LOG_VERBOSITY_WARNING, "Warning from add_single_var_to_rc() : Bad variable position %u", var->pos);
+            log_msg(LOG_VERBOSITY_WARNING,
+                    "Warning from add_single_var_to_rc() : Bad variable position %u",
+                    var->pos);
             return;
     }
 
-    log_msg(LOG_VERBOSITY_DEBUG, "add_single_var_to_rc() : Updating param (%u) %s to %s",
-                var->pos, var->name, val);
+    log_msg(LOG_VERBOSITY_DEBUG,
+            "add_single_var_to_rc() : Updating param (%u) %s to %s",
+            var->pos, var->name, val);
 
     fprintf(fhandle, RC_PARAM_TEMPLATE, var->name, val);
 }
@@ -1869,12 +1887,6 @@ validate_options(fko_cli_options_t *options)
             snprintf(options->http_user_agent, HTTP_MAX_USER_AGENT_LEN,
                 "%s%s", "Fwknop/", MY_VERSION);
 
-#if AFL_FUZZING
-    /* Don't issue IP resolution requests in AFL fuzzing mode
-    */
-    options->resolve_ip_http_https = 0;
-#endif
-
     if(options->http_proxy[0] != 0x0 && options->spa_proto != FKO_PROTO_HTTP)
     {
         log_msg(LOG_VERBOSITY_ERROR,
@@ -2272,6 +2284,9 @@ config_init(fko_cli_options_t *options, int argc, char **argv)
                 strlcpy(options->resolve_url, optarg, rlen);
                 add_var_to_bitmask(FWKNOP_CLI_ARG_RESOLVE_URL, &var_bitmask);
                 break;
+            case SERVER_RESOLVE_IPV4:
+                options->spa_server_resolve_ipv4 = 1;
+                break;
             case 'w':
                 if(options->wget_bin != NULL)
                     free(options->wget_bin);
@@ -2333,6 +2348,7 @@ config_init(fko_cli_options_t *options, int argc, char **argv)
             case GPG_HOME_DIR:
                 options->use_gpg = 1;
                 strlcpy(options->gpg_home_dir, optarg, sizeof(options->gpg_home_dir));
+                chop_char(options->gpg_home_dir, PATH_SEP);
                 add_var_to_bitmask(FWKNOP_CLI_ARG_USE_GPG, &var_bitmask);
                 add_var_to_bitmask(FWKNOP_CLI_ARG_GPG_HOMEDIR, &var_bitmask);
                 break;
@@ -2569,6 +2585,8 @@ usage(void)
       "                             $HOME/fwknop.run file\n"
       "     --rc-file               Specify path to the fwknop rc file (default\n"
       "                             is $HOME/.fwknoprc)\n"
+      "     --server-resolve-ipv4   Force IPv4 address resolution from DNS for\n"
+      "                             SPA server when using a hostname.\n"
       "     --save-rc-stanza        Save command line arguments to the\n"
       "                             $HOME/.fwknoprc stanza specified with the\n"
       "                             -n option.\n"
