@@ -4,7 +4,7 @@
  * @brief   The fwknop client.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -64,6 +64,14 @@ static int is_hostname_str_with_port(const char *str,
 static int enable_fault_injections(fko_cli_options_t * const opts);
 #endif
 
+#if AFL_FUZZING
+  /* These are used in AFL fuzzing mode so the fuzzing cycle is not
+   * interrupted by trying to read from stdin
+  */
+  #define AFL_ENC_KEY               "aflenckey"
+  #define AFL_HMAC_KEY              "aflhmackey"
+#endif
+
 #define NAT_ACCESS_STR_TEMPLATE     "%s,%d"             /*!< Template for a nat access string ip,port with sscanf*/
 #define HOSTNAME_BUFSIZE            64                  /*!< Maximum size of a hostname string */
 #define CTX_DUMP_BUFSIZE            4096                /*!< Maximum size allocated to a FKO context dump */
@@ -891,7 +899,8 @@ set_nat_access(fko_ctx_t ctx, fko_cli_options_t *options, const char * const acc
          * family to either AF_INET or AF_INET6 */
         hints.ai_family = AF_INET;
 
-        if (resolve_dest_adr(hostname, &hints, dst_ip_str, sizeof(dst_ip_str)) != 0)
+        if (resolve_dst_addr(hostname, &hints,
+                    dst_ip_str, sizeof(dst_ip_str), options) != 0)
         {
             log_msg(LOG_VERBOSITY_ERROR, "[*] Unable to resolve %s as an ip address",
                     hostname);
@@ -986,7 +995,7 @@ run_last_args(fko_cli_options_t *options, const char * const args_save_file)
 {
     FILE           *args_file_ptr = NULL;
     int             argc_new = 0, args_broken = 0;
-    char            args_str[MAX_LINE_LEN] = {0};
+    char            args_str[MAX_ARGS_LINE_LEN] = {0};
     char           *argv_new[MAX_CMDLINE_ARGS];  /* should be way more than enough */
 
     memset(argv_new, 0x0, sizeof(argv_new));
@@ -1005,7 +1014,7 @@ run_last_args(fko_cli_options_t *options, const char * const args_save_file)
         args_str[MAX_LINE_LEN-1] = '\0';
         if (options->verbose)
             log_msg(LOG_VERBOSITY_NORMAL, "Executing: %s", args_str);
-        if(strtoargv(args_str, argv_new, &argc_new, options) != 1)
+        if(strtoargv(args_str, argv_new, &argc_new) != 1)
         {
             args_broken = 1;
         }
@@ -1126,7 +1135,9 @@ static int
 get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
     char *key, int *key_len, char *hmac_key, int *hmac_key_len)
 {
+#if !AFL_FUZZING
     char   *key_tmp = NULL, *hmac_key_tmp = NULL;
+#endif
     int     use_hmac = 0, res = 0;
 
     memset(key, 0x0, MAX_KEY_LEN+1);
@@ -1173,6 +1184,9 @@ get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
                     "[+] GPG mode set, signing passphrase not required");
             else if(strlen(options->gpg_signer_key))
             {
+#if AFL_FUZZING
+                strlcpy(key, AFL_ENC_KEY, MAX_KEY_LEN+1);
+#else
                 key_tmp = getpasswd("Enter passphrase for signing: ", options->input_fd);
                 if(key_tmp == NULL)
                 {
@@ -1180,19 +1194,23 @@ get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
                     return 0;
                 }
                 strlcpy(key, key_tmp, MAX_KEY_LEN+1);
+#endif
                 *key_len = strlen(key);
             }
         }
         else
         {
+#if AFL_FUZZING
+            strlcpy(key, AFL_ENC_KEY, MAX_KEY_LEN+1);
+#else
             key_tmp = getpasswd("Enter encryption key: ", options->input_fd);
-
             if(key_tmp == NULL)
             {
                 log_msg(LOG_VERBOSITY_ERROR, "[*] getpasswd() key error.");
                 return 0;
             }
             strlcpy(key, key_tmp, MAX_KEY_LEN+1);
+#endif
             *key_len = strlen(key);
         }
     }
@@ -1232,15 +1250,17 @@ get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
         }
         else
         {
+#if AFL_FUZZING
+            strlcpy(hmac_key, AFL_HMAC_KEY, MAX_KEY_LEN+1);
+#else
             hmac_key_tmp = getpasswd("Enter HMAC key: ", options->input_fd);
-
             if(hmac_key_tmp == NULL)
             {
                 log_msg(LOG_VERBOSITY_ERROR, "[*] getpasswd() key error.");
                 return 0;
             }
-
             strlcpy(hmac_key, hmac_key_tmp, MAX_KEY_LEN+1);
+#endif
             *hmac_key_len = strlen(hmac_key);
             use_hmac = 1;
         }
