@@ -6,7 +6,7 @@
  * Purpose: Network-related functions for the fwknop client
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -54,7 +54,7 @@ dump_transmit_options(const fko_cli_options_t *options)
 
 /* Function to generate a header checksum.
 */
-unsigned short
+static unsigned short
 chksum(unsigned short *buf, int nbytes)
 {
     unsigned int   sum;
@@ -86,8 +86,8 @@ static int
 send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
     const fko_cli_options_t *options)
 {
-    int     sock, res=0, error;
-    struct  addrinfo *result, *rp, hints;
+    int     sock=-1, sock_success=0, res=0, error;
+    struct  addrinfo *result=NULL, *rp, hints;
     char    port_str[MAX_PORT_STR_LEN+1] = {0};
 
     if (options->test)
@@ -119,6 +119,14 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
 
     snprintf(port_str, MAX_PORT_STR_LEN+1, "%d", options->spa_dst_port);
 
+#if AFL_FUZZING
+    /* Make sure to never send SPA packets under AFL fuzzing cycles
+    */
+    log_msg(LOG_VERBOSITY_NORMAL,
+        "AFL fuzzing enabled, SPA packet not actually sent.");
+    return res;
+#endif
+
     error = getaddrinfo(options->spa_server_str, port_str, &hints, &result);
 
     if (error != 0)
@@ -128,30 +136,46 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
     }
 
     for (rp = result; rp != NULL; rp = rp->ai_next) {
+        /* Apply --server-resolve-ipv4 criteria
+        */
+        if(options->spa_server_resolve_ipv4)
+        {
+            if(rp->ai_family != AF_INET)
+            {
+                log_msg(LOG_VERBOSITY_DEBUG, "Non-IPv4 resolution");
+                continue;
+            }
+        }
+
         sock = socket(rp->ai_family, rp->ai_socktype,
                 rp->ai_protocol);
         if (sock < 0)
             continue;
 
         if ((error = (connect(sock, rp->ai_addr, rp->ai_addrlen) != -1)))
+        {
+            sock_success = 1;
             break;  /* made it */
-
+        }
+        else /* close the open socket if there was a connect error */
+        {
 #ifdef WIN32
-        closesocket(sock);
+            closesocket(sock);
 #else
-        close(sock);
+            close(sock);
 #endif
+        }
     }
+    if(result != NULL)
+        freeaddrinfo(result);
 
-    if (rp == NULL) {
+    if (! sock_success) {
         log_msg(LOG_VERBOSITY_ERROR,
                 "send_spa_packet_tcp_or_udp: Could not create socket: ",
                 strerror(errno));
         return -1;
     }
 
-    freeaddrinfo(result);
-
     res = send(sock, spa_data, sd_len, 0);
 
     if(res < 0)
@@ -256,7 +280,7 @@ send_spa_packet_tcp_raw(const char *spa_data, const int sd_len,
     tcph->check     = 0;
     tcph->urg_ptr   = 0;
 
-    /* No we can compute our checksum.
+    /* Now we can compute our checksum.
     */
     iph->check = chksum((unsigned short *)pkt_data, iph->tot_len);
 
@@ -355,7 +379,7 @@ send_spa_packet_udp_raw(const char *spa_data, const int sd_len,
     udph->check     = 0;
     udph->len       = htons(sd_len + sizeof(struct udphdr));
 
-    /* No we can compute our checksum.
+    /* Now we can compute our checksum.
     */
     iph->check = chksum((unsigned short *)pkt_data, iph->tot_len);
 
@@ -459,7 +483,7 @@ send_spa_packet_icmp(const char *spa_data, const int sd_len,
         icmph->un.echo.sequence = htons(1);
     }
 
-    /* No we can compute our checksum.
+    /* Now we can compute our checksum.
     */
     iph->check = chksum((unsigned short *)pkt_data, iph->tot_len);
     icmph->checksum = chksum((unsigned short *)icmph, sizeof(struct icmphdr) + sd_len);
@@ -585,6 +609,9 @@ send_spa_packet_http(const char *spa_data, const int sd_len,
         return 0;
     }
 
+    /* In AFL fuzzing mode, the following function will not send
+     * the SPA packet.
+    */
     return send_spa_packet_tcp_or_udp(http_buf, strlen(http_buf), options);
 }
 
@@ -679,7 +706,16 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
          * the ip address */
         hints.ai_family = AF_INET;
 
-        if (resolve_dest_adr(options->spa_server_str, &hints, ip_str, sizeof(ip_str)) != 0)
+#if AFL_FUZZING
+        /* Make sure to never send SPA packets under AFL fuzzing cycles
+        */
+        log_msg(LOG_VERBOSITY_NORMAL,
+            "AFL fuzzing enabled, SPA packet not actually sent.");
+        return res;
+#endif
+
+        if (resolve_dst_addr(options->spa_server_str,
+                    &hints, ip_str, sizeof(ip_str), options) != 0)
         {
             log_msg(LOG_VERBOSITY_ERROR, "[*] Unable to resolve %s as an ip address",
                     options->spa_server_str);
