@@ -6,7 +6,7 @@
  * Purpose: General/Generic functions for the fwknop client.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -148,17 +148,19 @@ get_in_addr(struct sockaddr *sa)
 }
 
 /**
- * @brief  Resolve a domain name as an ip adress.
+ * @brief  Resolve a domain name as an IP address.
  *
  * @param dns_str    Name of the host to resolve.
  * @param hints      Hints to reduce the number of result from getaddrinfo()
  * @param ip_str     String where to store the resolve ip address
  * @param ip_bufsize Number of bytes available in the ip_str buffer
+ * @param opts       Client command line options
  *
  * @return 0 if successful, 1 if an error occured.
  */
 int
-resolve_dest_adr(const char *dns_str, struct addrinfo *hints, char *ip_str, size_t ip_bufsize)
+resolve_dst_addr(const char *dns_str, struct addrinfo *hints,
+        char *ip_str, size_t ip_bufsize, fko_cli_options_t *opts)
 {
     int                 error;      /* Function error return code */
     struct addrinfo    *result;     /* Result of getaddrinfo() */
@@ -173,7 +175,7 @@ resolve_dest_adr(const char *dns_str, struct addrinfo *hints, char *ip_str, size
     /* Try to resolve the host name */
     error = getaddrinfo(dns_str, NULL, hints, &result);
     if (error != 0)
-        fprintf(stderr, "resolve_dest_adr() : %s\n", gai_strerror(error));
+        fprintf(stderr, "resolve_dst_addr() : %s\n", gai_strerror(error));
 
     else
     {
@@ -182,6 +184,17 @@ resolve_dest_adr(const char *dns_str, struct addrinfo *hints, char *ip_str, size
         /* Go through the linked list of addrinfo structures */
         for (rp = result; rp != NULL; rp = rp->ai_next)
         {
+            /* Apply --server-resolve-ipv4 criteria
+            */
+            if(opts->spa_server_resolve_ipv4)
+            {
+                if(rp->ai_family != AF_INET)
+                {
+                    log_msg(LOG_VERBOSITY_DEBUG, "Non-IPv4 resolution");
+                    continue;
+                }
+            }
+
             memset(ip_str, 0, ip_bufsize);
 #if WIN32 && WINVER <= 0x0600
 			/* On older Windows systems (anything before Vista?),
@@ -195,12 +208,12 @@ resolve_dest_adr(const char *dns_str, struct addrinfo *hints, char *ip_str, size
             sai_remote = (struct sockaddr_in *)get_in_addr((struct sockaddr *)(rp->ai_addr));
             if (inet_ntop(rp->ai_family, sai_remote, ip_str, ip_bufsize) != NULL)
 #endif
-			{
+            {
                 error = 0;
                 break;
             }
             else
-                log_msg(LOG_VERBOSITY_ERROR, "resolve_dest_adr() : inet_ntop (%d) - %s",
+                log_msg(LOG_VERBOSITY_ERROR, "resolve_dst_addr() : inet_ntop (%d) - %s",
                         errno, strerror(errno));
         }
 
