@@ -6,7 +6,7 @@
  * Purpose: Set/Get the current username.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -55,8 +55,16 @@ fko_set_username(fko_ctx_t ctx, const char * const spoof_user)
     /* If spoof_user was not passed in, check for a SPOOF_USER enviroment
      * variable.  If it is set, use its value.
     */
-    if(spoof_user != NULL && strnlen(spoof_user, MAX_SPA_USERNAME_SIZE))
-        username = (char*)spoof_user;
+    if(spoof_user != NULL && spoof_user[0] != '\0')
+    {
+#if HAVE_LIBFIU
+        fiu_return_on("fko_set_username_strdup", FKO_ERROR_MEMORY_ALLOCATION);
+#endif
+        username = strdup(spoof_user);
+        if(username == NULL)
+            return(FKO_ERROR_MEMORY_ALLOCATION);
+        is_user_heap_allocated = 1;
+    }
     else
         username = getenv("SPOOF_USER");
 
@@ -82,9 +90,6 @@ fko_set_username(fko_ctx_t ctx, const char * const spoof_user)
             {
                 if((username = getenv("USER")) == NULL)
                 {
-#if HAVE_LIBFIU
-                    fiu_return_on("fko_set_username_strdup1", FKO_ERROR_MEMORY_ALLOCATION);
-#endif
                     username = strdup("NO_USER");
                     if(username == NULL)
                         return(FKO_ERROR_MEMORY_ALLOCATION);
@@ -115,10 +120,6 @@ fko_set_username(fko_ctx_t ctx, const char * const spoof_user)
     if(ctx->username != NULL)
         free(ctx->username);
 
-#if HAVE_LIBFIU
-    fiu_return_on("fko_set_username_strdup2", FKO_ERROR_MEMORY_ALLOCATION);
-#endif
-
     ctx->username = strdup(username);
 
     ctx->state |= FKO_DATA_MODIFIED;
