@@ -6,7 +6,7 @@
  * Purpose: Command-line and config file processing for fwknop server.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -45,12 +45,12 @@
 /* Check to see if an integer variable has a value that is within a
  * specific range
 */
-static void
+static int
 range_check(fko_srv_options_t *opts, char *var, char *val, int low, int high)
 {
-    int     is_err;
+    int     is_err, rv;
 
-    strtol_wrapper(val, low, high, NO_EXIT_UPON_ERR, &is_err);
+    rv = strtol_wrapper(val, low, high, NO_EXIT_UPON_ERR, &is_err);
     if(is_err != FKO_SUCCESS)
     {
         log_msg(LOG_ERR, "[*] var %s value '%s' not in the range %d-%d",
@@ -58,7 +58,7 @@ range_check(fko_srv_options_t *opts, char *var, char *val, int low, int high)
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
-    return;
+    return rv;
 }
 
 /* Take an index and a string value. malloc the space for the value
@@ -143,18 +143,30 @@ validate_int_var_ranges(fko_srv_options_t *opts)
     int     is_err = FKO_SUCCESS;
 #endif
 
-    range_check(opts, "PCAP_LOOP_SLEEP", opts->config[CONF_PCAP_LOOP_SLEEP],
-        1, RCHK_MAX_PCAP_LOOP_SLEEP);
-    range_check(opts, "MAX_SPA_PACKET_AGE", opts->config[CONF_MAX_SPA_PACKET_AGE],
-        1, RCHK_MAX_SPA_PACKET_AGE);
-    range_check(opts, "MAX_SNIFF_BYTES", opts->config[CONF_MAX_SNIFF_BYTES],
-        1, RCHK_MAX_SNIFF_BYTES);
-    range_check(opts, "TCPSERV_PORT", opts->config[CONF_TCPSERV_PORT],
-        1, RCHK_MAX_TCPSERV_PORT);
-    range_check(opts, "UDPSERV_PORT", opts->config[CONF_UDPSERV_PORT],
-        1, RCHK_MAX_UDPSERV_PORT);
-    range_check(opts, "UDPSERV_PORT", opts->config[CONF_UDPSERV_SELECT_TIMEOUT],
-        1, RCHK_MAX_UDPSERV_SELECT_TIMEOUT);
+    opts->pcap_loop_sleep = range_check(opts,
+            "PCAP_LOOP_SLEEP", opts->config[CONF_PCAP_LOOP_SLEEP],
+            1, RCHK_MAX_PCAP_LOOP_SLEEP);
+    opts->pcap_dispatch_count = range_check(opts,
+            "PCAP_DISPATCH_COUNT", opts->config[CONF_PCAP_DISPATCH_COUNT],
+            1, RCHK_MAX_PCAP_DISPATCH_COUNT);
+    opts->max_spa_packet_age = range_check(opts,
+            "MAX_SPA_PACKET_AGE", opts->config[CONF_MAX_SPA_PACKET_AGE],
+            1, RCHK_MAX_SPA_PACKET_AGE);
+    opts->max_sniff_bytes = range_check(opts,
+            "MAX_SNIFF_BYTES", opts->config[CONF_MAX_SNIFF_BYTES],
+            1, RCHK_MAX_SNIFF_BYTES);
+    opts->rules_chk_threshold = range_check(opts,
+            "RULES_CHECK_THRESHOLD", opts->config[CONF_RULES_CHECK_THRESHOLD],
+            0, RCHK_MAX_RULES_CHECK_THRESHOLD);
+    opts->tcpserv_port = range_check(opts,
+            "TCPSERV_PORT", opts->config[CONF_TCPSERV_PORT],
+            1, RCHK_MAX_TCPSERV_PORT);
+    opts->udpserv_port = range_check(opts,
+            "UDPSERV_PORT", opts->config[CONF_UDPSERV_PORT],
+            1, RCHK_MAX_UDPSERV_PORT);
+    opts->udpserv_select_timeout = range_check(opts,
+            "UDPSERV_SELECT_TIMEOUT", opts->config[CONF_UDPSERV_SELECT_TIMEOUT],
+            1, RCHK_MAX_UDPSERV_SELECT_TIMEOUT);
 
 #if FIREWALL_IPFW
     range_check(opts, "IPFW_START_RULE_NUM", opts->config[CONF_IPFW_START_RULE_NUM],
@@ -201,6 +213,69 @@ validate_int_var_ranges(fko_srv_options_t *opts)
     return;
 }
 
+/**
+ * @brief Generate Rijndael + HMAC keys from /dev/urandom (base64 encoded).
+ *
+ * @param options FKO command line option structure
+ */
+static void
+generate_keys(fko_srv_options_t *options)
+{
+    char key_base64[MAX_B64_KEY_LEN+1];
+    char hmac_key_base64[MAX_B64_KEY_LEN+1];
+
+    FILE  *key_gen_file_ptr = NULL;
+    int res;
+
+    /* Set defaults and validate for --key-gen mode
+    */
+    if(options->key_len == 0)
+        options->key_len = FKO_DEFAULT_KEY_LEN;
+
+    if(options->hmac_key_len == 0)
+        options->hmac_key_len = FKO_DEFAULT_HMAC_KEY_LEN;
+
+    if(options->hmac_type == 0)
+        options->hmac_type = FKO_DEFAULT_HMAC_MODE;
+
+    /* Zero out the key buffers */
+    memset(key_base64, 0x00, sizeof(key_base64));
+    memset(hmac_key_base64, 0x00, sizeof(hmac_key_base64));
+
+    /* Generate the key through libfko */
+    res = fko_key_gen(key_base64, options->key_len,
+            hmac_key_base64, options->hmac_key_len,
+            options->hmac_type);
+
+    if(res != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "%s: fko_key_gen: Error %i - %s",
+            MY_NAME, res, fko_errstr(res));
+        clean_exit(options, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    if(options->key_gen_file[0] != '\0')
+    {
+        if ((key_gen_file_ptr = fopen(options->key_gen_file, "w")) == NULL)
+        {
+            log_msg(LOG_ERR, "Unable to create key gen file: %s: %s",
+                options->key_gen_file, strerror(errno));
+            clean_exit(options, NO_FW_CLEANUP, EXIT_FAILURE);
+        }
+        fprintf(key_gen_file_ptr, "KEY_BASE64: %s\nHMAC_KEY_BASE64: %s\n",
+            key_base64, hmac_key_base64);
+        fclose(key_gen_file_ptr);
+        fprintf(stdout, "[+] Wrote Rijndael and HMAC keys to: %s",
+            options->key_gen_file);
+    }
+    else
+    {
+        fprintf(stdout, "KEY_BASE64: %s\nHMAC_KEY_BASE64: %s\n",
+                key_base64, hmac_key_base64);
+    }
+    clean_exit(options, NO_FW_CLEANUP, EXIT_SUCCESS);
+}
+
 /* Parse the config file...
 */
 static void
@@ -394,6 +469,13 @@ validate_options(fko_srv_options_t *opts)
         set_config_entry(opts, CONF_PCAP_LOOP_SLEEP,
             DEF_PCAP_LOOP_SLEEP);
 
+    /* Control whether to exit if the interface where we're sniffing
+     * goes down.
+    */
+    if(opts->config[CONF_EXIT_AT_INTF_DOWN] == NULL)
+        set_config_entry(opts, CONF_EXIT_AT_INTF_DOWN,
+            DEF_EXIT_AT_INTF_DOWN);
+
     /* PCAP Filter.
     */
     if(opts->config[CONF_PCAP_FILTER] == NULL)
@@ -421,13 +503,20 @@ validate_options(fko_srv_options_t *opts)
         set_config_entry(opts, CONF_MAX_SPA_PACKET_AGE,
             DEF_MAX_SPA_PACKET_AGE);
 
-
     /* Enable digest persistence.
     */
     if(opts->config[CONF_ENABLE_DIGEST_PERSISTENCE] == NULL)
         set_config_entry(opts, CONF_ENABLE_DIGEST_PERSISTENCE,
             DEF_ENABLE_DIGEST_PERSISTENCE);
-            
+
+    /* Set firewall rule "deep" collection interval - this allows
+     * fwknopd to remove rules with proper _exp_<time> expiration
+     * times even when added by a different program.
+    */
+    if(opts->config[CONF_RULES_CHECK_THRESHOLD] == NULL)
+        set_config_entry(opts, CONF_RULES_CHECK_THRESHOLD,
+            DEF_RULES_CHECK_THRESHOLD);
+
     /* Enable destination rule.
     */
     if(opts->config[CONF_ENABLE_DESTINATION_RULE] == NULL)
@@ -801,6 +890,11 @@ validate_options(fko_srv_options_t *opts)
     if(opts->config[CONF_GPG_EXE] == NULL)
         set_config_entry(opts, CONF_GPG_EXE, DEF_GPG_EXE);
 
+    /* sudo executable
+    */
+    if(opts->config[CONF_SUDO_EXE] == NULL)
+        set_config_entry(opts, CONF_SUDO_EXE, DEF_SUDO_EXE);
+
     /* Enable SPA over HTTP.
     */
     if(opts->config[CONF_ENABLE_SPA_OVER_HTTP] == NULL)
@@ -926,10 +1020,76 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
     */
     optind = 0;
 
-    /* First, scan the command-line args for -h/--help or an alternate
+    /* Enable access.conf parsing (in case this is a re-config)
+    */
+    enable_acc_stanzas_init();
+
+    /* First, scan the command-line args to see if we are in key-generation
+     * mode. This is independent of config parsing and other operations, so
+     * it is done as the very first thing. Also handle printing of the fwknop
+     * version string since we don't need to parse a config for this.
+    */
+    while ((cmd_arg = getopt_long(argc, argv,
+            GETOPTS_OPTION_STRING, cmd_opts, &index)) != -1) {
+
+        switch(cmd_arg) {
+            case 'V':
+                fprintf(stdout, "fwknopd server %s, compiled for firewall bin: %s\n",
+                        MY_VERSION, FIREWALL_EXE);
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+            case 'k':
+                opts->key_gen = 1;
+                break;
+            case KEY_GEN_FILE:
+                opts->key_gen = 1;
+                strlcpy(opts->key_gen_file, optarg, sizeof(opts->key_gen_file));
+                break;
+            case KEY_LEN:  /* used in --key-gen mode only */
+                opts->key_len = strtol_wrapper(optarg, 1,
+                        MAX_KEY_LEN, NO_EXIT_UPON_ERR, &is_err);
+                if(is_err != FKO_SUCCESS)
+                {
+                    log_msg(LOG_ERR,
+                            "Invalid key length '%s', must be in [%d-%d]",
+                            optarg, 1, MAX_KEY_LEN);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case HMAC_DIGEST_TYPE:  /* used in --key-gen mode only */
+                if((opts->hmac_type = hmac_digest_strtoint(optarg)) < 0)
+                {
+                    log_msg(LOG_ERR,
+                        "* Invalid hmac digest type: %s, use {md5,sha1,sha256,sha384,sha512}",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case HMAC_KEY_LEN:  /* used in --key-gen mode only */
+                opts->hmac_key_len = strtol_wrapper(optarg, 1,
+                        MAX_KEY_LEN, NO_EXIT_UPON_ERR, &is_err);
+                if(is_err != FKO_SUCCESS)
+                {
+                    log_msg(LOG_ERR,
+                            "Invalid hmac key length '%s', must be in [%d-%d]",
+                            optarg, 1, MAX_KEY_LEN);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+        }
+    }
+
+    if(opts->key_gen)
+        generate_keys(opts); /* this function exits */
+
+    /* Reset the options index so we can run through them again.
+    */
+    optind = 0;
+
+    /* Now, scan the command-line args for -h/--help or an alternate
      * configuration file. If we find an alternate config file, use it,
-     * otherwise use the default.  We also grab any override config files
-     * as well.
+     * otherwise use the default. We also grab any override config files
+     * as well. In addition, we handle key generation here since this is
+     * independent of configuration parsing.
     */
     while ((cmd_arg = getopt_long(argc, argv,
             GETOPTS_OPTION_STRING, cmd_opts, &index)) != -1) {
@@ -1054,7 +1214,26 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
 #endif
                 break;
             case 'a':
-                set_config_entry(opts, CONF_ACCESS_FILE, optarg);
+                if (is_valid_file(optarg))
+                    set_config_entry(opts, CONF_ACCESS_FILE, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] Invalid access.conf file path '%s'", optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case ACCESS_FOLDER:
+                chop_char(optarg, PATH_SEP);
+                if (is_valid_dir(optarg))
+                    set_config_entry(opts, CONF_ACCESS_FOLDER, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] Invalid access folder directory '%s' could not lstat(), does not exist, or too large?",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
                 break;
             case 'c':
                 /* This was handled earlier */
@@ -1087,6 +1266,10 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
                 opts->exit_after_parse_config = 1;
                 opts->foreground = 1;
                 break;
+            case EXIT_VALIDATE_DIGEST_CACHE:
+                opts->exit_parse_digest_cache = 1;
+                opts->foreground = 1;
+                break;
             case 'f':
                 opts->foreground = 1;
                 break;
@@ -1108,15 +1291,25 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case FW_FLUSH:
                 opts->fw_flush = 1;
                 break;
-            case GPG_HOME_DIR:
-                if (is_valid_dir(optarg))
-                {
-                    set_config_entry(opts, CONF_GPG_HOME_DIR, optarg);
-                }
+            case GPG_EXE_PATH:
+                if (is_valid_exe(optarg))
+                    set_config_entry(opts, CONF_GPG_EXE, optarg);
                 else
                 {
                     log_msg(LOG_ERR,
-                        "[*] Directory '%s' could not stat()/does not exist?",
+                        "[*] gpg path '%s' could not lstat()/not executable?",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case GPG_HOME_DIR:
+                chop_char(optarg, PATH_SEP);
+                if (is_valid_dir(optarg))
+                    set_config_entry(opts, CONF_GPG_HOME_DIR, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] gpg home directory '%s' could not lstat(), does not exist, or too large?",
                         optarg);
                     clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
                 }
@@ -1146,7 +1339,14 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
                 set_config_entry(opts, CONF_PCAP_FILTER, optarg);
                 break;
             case PCAP_FILE:
-                set_config_entry(opts, CONF_PCAP_FILE, optarg);
+                if (is_valid_file(optarg))
+                    set_config_entry(opts, CONF_PCAP_FILE, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] Invalid pcap file path '%s'", optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
                 break;
             case ENABLE_PCAP_ANY_DIRECTION:
                 opts->pcap_any_direction = 1;
@@ -1163,6 +1363,17 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'S':
                 opts->status = 1;
                 break;
+            case SUDO_EXE_PATH:
+                if (is_valid_exe(optarg))
+                    set_config_entry(opts, CONF_SUDO_EXE, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] sudo path '%s' could not stat()/not executable?",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
             case 't':
                 opts->test = 1;
                 break;
@@ -1176,9 +1387,6 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case SYSLOG_ENABLE:
                 opts->syslog_enable = 1;
                 break;
-            case 'V':
-                fprintf(stdout, "fwknopd server %s\n", MY_VERSION);
-                clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
             default:
                 usage();
                 clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
@@ -1223,6 +1431,8 @@ usage(void)
     fprintf(stdout,
       "Usage: fwknopd [options]\n\n"
       " -a, --access-file       - Specify an alternate access.conf file.\n"
+      "     --access-folder     - Specify an access.conf folder.  All .conf\n"
+      "                           files in this folder will be processed.\n"
       " -c, --config-file       - Specify an alternate configuration file.\n"
       " -f, --foreground        - Run fwknopd in the foreground (do not become\n"
       "                           a background daemon).\n"
@@ -1261,6 +1471,7 @@ usage(void)
       " --dump-serv-err-codes   - List all server error codes (only needed by the\n"
       "                           test suite).\n"
       " --exit-parse-config     - Parse config files and exit.\n"
+      " --exit-parse-digest-cache - Parse and validate digest cache  and exit.\n"
       " --fault-injection-tag   - Enable a fault injection tag (only needed by the\n"
       "                           test suite).\n"
       " --pcap-file             - Read potential SPA packets from an existing pcap\n"
@@ -1279,6 +1490,8 @@ usage(void)
       "                           done in the access.conf file).\n"
       "     --gpg-exe           - Specify the path to GPG (this is normally done in\n"
       "                           the access.conf file).\n"
+      "     --sudo-exe          - Specify the path to sudo (the default path is\n"
+      "                           /usr/bin/sudo).\n"
       " --no-firewd-check-support\n"
       "                         - Disable test for 'firewall-cmd ... -C' support.\n"
       " --no-ipt-check-support  - Disable test for 'iptables -C' support.\n"
