@@ -6,7 +6,7 @@
  * Purpose: Header file for fw_util.c.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -66,7 +66,8 @@
 int fw_config_init(fko_srv_options_t * const opts);
 int fw_initialize(const fko_srv_options_t * const opts);
 int fw_cleanup(const fko_srv_options_t * const opts);
-void check_firewall_rules(const fko_srv_options_t * const opts);
+void check_firewall_rules(const fko_srv_options_t * const opts,
+        const int chk_rm_all);
 int fw_dump_rules(const fko_srv_options_t * const opts);
 int process_spa_request(const fko_srv_options_t * const opts,
         const acc_stanza_t * const acc, spa_data_t * const spadat);
