@@ -6,7 +6,7 @@
  * Purpose: Fwknop routines for managing ipfw firewall rules.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -601,7 +601,8 @@ process_spa_request(const fko_srv_options_t * const opts,
  * firewall rules.
 */
 void
-check_firewall_rules(const fko_srv_options_t * const opts)
+check_firewall_rules(const fko_srv_options_t * const opts,
+        const int chk_rm_all)
 {
     char            exp_str[12]     = {0};
     char            rule_num_str[6] = {0};
@@ -639,6 +640,7 @@ check_firewall_rules(const fko_srv_options_t * const opts)
 
     res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE,
                 WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
+    chop_newline(cmd_out);
 
     log_msg(LOG_DEBUG, "check_firewall_rules() CMD: '%s' (res: %d)",
         cmd_buf, res);
@@ -681,6 +683,15 @@ check_firewall_rules(const fko_srv_options_t * const opts)
         tmp_mark = ndx;
 
         strlcpy(exp_str, ndx, sizeof(exp_str));
+        chop_spaces(exp_str);
+        if(!is_digits(exp_str))
+        {
+            /* go to the next rule if it exists
+            */
+            ndx = strstr(tmp_mark, EXPIRE_COMMENT_PREFIX);
+            continue;
+        }
+
         rule_exp = (time_t)atoll(exp_str);
 
         if(rule_exp <= now)
