@@ -6,7 +6,7 @@
  * Purpose: An implementation of an fwknop server.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -76,6 +76,7 @@ int
 main(int argc, char **argv)
 {
     fko_srv_options_t   opts;
+    int depth = 0;
 
     while(1)
     {
@@ -148,9 +149,27 @@ main(int argc, char **argv)
             clean_exit(&opts, FW_CLEANUP, EXIT_SUCCESS);
         }
 
-        /* Process the access.conf file.
+        if (opts.config[CONF_ACCESS_FOLDER] != NULL) //If we have an access folder, process it
+        {
+            if (parse_access_folder(&opts, opts.config[CONF_ACCESS_FOLDER], &depth) != EXIT_SUCCESS)
+            {
+                clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
+            }
+        }
+        /* Process the access.conf file, but only if no access.conf folder was specified.
         */
-        parse_access_file(&opts);
+        else if (parse_access_file(&opts, opts.config[CONF_ACCESS_FILE], &depth) != EXIT_SUCCESS)
+        {
+            clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
+        }
+
+        /* We must have at least one valid access stanza at this point
+        */
+        if(! valid_access_stanzas(opts.acc_stanzas))
+        {
+            log_msg(LOG_ERR, "Fatal, could not find any valid access.conf stanzas");
+            clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
+        }
 
         /* Show config (including access.conf vars) and exit dump config was
          * wanted.
@@ -162,10 +181,19 @@ main(int argc, char **argv)
             clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
         }
 
+        /* Now is the right time to bail if we're just parsing the configs
+        */
+        if(opts.exit_after_parse_config)
+        {
+            log_msg(LOG_INFO, "Configs parsed, exiting.");
+            clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+        }
+
         /* Acquire pid, become a daemon or run in the foreground, write pid
          * to pid file.
         */
-        setup_pid(&opts);
+        if(! opts.exit_parse_digest_cache)
+            setup_pid(&opts);
 
         if(opts.verbose > 1 && opts.foreground)
         {
@@ -179,9 +207,9 @@ main(int argc, char **argv)
         */
         init_digest_cache(&opts);
 
-        if(opts.exit_after_parse_config)
+        if(opts.exit_parse_digest_cache)
         {
-            log_msg(LOG_INFO, "Configs parsed, exiting.");
+            log_msg(LOG_INFO, "Digest cache parsed, exiting.");
             clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
         }
 
@@ -203,7 +231,7 @@ main(int argc, char **argv)
         /* Prepare the firewall - i.e. flush any old rules and (for iptables)
          * create fwknop chains.
         */
-        if(!opts.test && (fw_initialize(&opts) != 1))
+        if(!opts.test && opts.enable_fw && (fw_initialize(&opts) != 1))
             clean_exit(&opts, FW_CLEANUP, EXIT_FAILURE);
 
         /* If we are to acquire SPA data via a UDP socket, start it up here.
@@ -224,7 +252,11 @@ main(int argc, char **argv)
 
         /* If the TCP server option was set, fire it up here. Note that in
          * this mode, fwknopd still acquires SPA packets via libpcap. If you
-         * want to use UDP only without the libpcap dependency, see the FIXME...
+         * want to use UDP only without the libpcap dependency, then fwknop
+         * needs to be compiled with --enable-udp-server. Note that the UDP
+         * server can be run even when fwknopd links against libpcap as well,
+         * but there is no reason to link against it if SPA packets are
+         * always going to be acquired via a UDP socket.
         */
         if(strncasecmp(opts.config[CONF_ENABLE_TCP_SERVER], "Y", 1) == 0)
         {
@@ -238,8 +270,11 @@ main(int argc, char **argv)
 #if USE_LIBPCAP
         /* Intiate pcap capture mode...
         */
-        if(strncasecmp(opts.config[CONF_ENABLE_UDP_SERVER], "N", 1) == 0)
+        if(!opts.enable_udp_server
+                && strncasecmp(opts.config[CONF_ENABLE_UDP_SERVER], "N", 1) == 0)
+        {
             pcap_capture(&opts);
+        }
 #endif
 
         /* Deal with any signals that we've received and break out
@@ -549,7 +584,8 @@ static int handle_signals(fko_srv_options_t *opts)
         {
             log_msg(LOG_WARNING, "Got SIGHUP. Re-reading configs.");
             free_configs(opts);
-            kill(opts->tcp_server_pid, SIGTERM);
+            if(opts->tcp_server_pid > 0)
+                kill(opts->tcp_server_pid, SIGTERM);
             usleep(1000000);
             got_sighup = 0;
             rv = 0;  /* this means fwknopd will not exit */
@@ -742,18 +778,15 @@ static int
 make_dir_path(const char * const run_dir)
 {
     struct stat     st;
-    int             res = 0, len = 0;
+    int             res = 0;
     char            tmp_path[MAX_PATH_LEN];
     char            *ndx;
 
     strlcpy(tmp_path, run_dir, sizeof(tmp_path));
 
-    len = strlen(tmp_path);
-
     /* Strip any trailing dir sep char.
     */
-    if(tmp_path[len-1] == PATH_SEP)
-        tmp_path[len-1] = '\0';
+    chop_char(tmp_path, PATH_SEP);
 
     for(ndx = tmp_path+1; *ndx; ndx++)
     {
