@@ -6,7 +6,7 @@
  * Purpose: Header file for fwknopd source files.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -88,9 +88,11 @@
 #define DEF_PCAP_DISPATCH_COUNT         "100"
 #define DEF_PCAP_LOOP_SLEEP             "100000" /* a tenth of a second (in microseconds) */
 #define DEF_ENABLE_PCAP_ANY_DIRECTION   "N"
+#define DEF_EXIT_AT_INTF_DOWN           "Y"
 #define DEF_ENABLE_SPA_PACKET_AGING     "Y"
 #define DEF_MAX_SPA_PACKET_AGE          "120"
 #define DEF_ENABLE_DIGEST_PERSISTENCE   "Y"
+#define DEF_RULES_CHECK_THRESHOLD       "20"
 #define DEF_MAX_SNIFF_BYTES             "1500"
 #define DEF_GPG_HOME_DIR                "/root/.gnupg"
 #ifdef  GPG_EXE
@@ -98,6 +100,11 @@
 #else
   #define DEF_GPG_EXE                   "/usr/bin/gpg"
 #endif
+#ifdef  SUDO_EXE
+  #define DEF_SUDO_EXE                   SUDO_EXE
+#else
+  #define DEF_SUDO_EXE                   "/usr/bin/sudo"
+#endif
 #define DEF_ENABLE_SPA_OVER_HTTP        "N"
 #define DEF_ENABLE_TCP_SERVER           "N"
 #define DEF_TCPSERV_PORT                "62201"
@@ -123,7 +130,10 @@
 #define RCHK_MAX_UDPSERV_PORT           ((2 << 16) - 1)
 #define RCHK_MAX_UDPSERV_SELECT_TIMEOUT (2 << 22)
 #define RCHK_MAX_PCAP_DISPATCH_COUNT    (2 << 22)
-#define RCHK_MAX_FW_TIMEOUT             (2 << 22)
+#define RCHK_MAX_FW_TIMEOUT             (2 << 22) /* seconds */
+#define RCHK_MAX_CMD_CYCLE_TIMER        (2 << 22) /* seconds */
+#define RCHK_MIN_CMD_CYCLE_TIMER        1
+#define RCHK_MAX_RULES_CHECK_THRESHOLD  ((2 << 16) - 1)
 
 /* FirewallD-specific defines
 */
@@ -132,7 +142,7 @@
   #define DEF_FLUSH_FIREWD_AT_INIT         "Y"
   #define DEF_FLUSH_FIREWD_AT_EXIT         "Y"
   #define DEF_ENABLE_FIREWD_FORWARDING     "N"
-  #define DEF_ENABLE_FIREWD_LOCAL_NAT      "Y"
+  #define DEF_ENABLE_FIREWD_LOCAL_NAT      "N"
   #define DEF_ENABLE_FIREWD_SNAT           "N"
   #define DEF_ENABLE_FIREWD_OUTPUT         "N"
   #define DEF_ENABLE_FIREWD_COMMENT_CHECK  "Y"
@@ -152,7 +162,7 @@
   #define DEF_FLUSH_IPT_AT_INIT         "Y"
   #define DEF_FLUSH_IPT_AT_EXIT         "Y"
   #define DEF_ENABLE_IPT_FORWARDING     "N"
-  #define DEF_ENABLE_IPT_LOCAL_NAT      "Y"
+  #define DEF_ENABLE_IPT_LOCAL_NAT      "N"
   #define DEF_ENABLE_IPT_SNAT           "N"
   #define DEF_ENABLE_IPT_OUTPUT         "N"
   #define DEF_ENABLE_IPT_COMMENT_CHECK  "Y"
@@ -226,10 +236,12 @@ enum {
     CONF_PCAP_DISPATCH_COUNT,
     CONF_PCAP_LOOP_SLEEP,
     CONF_ENABLE_PCAP_ANY_DIRECTION,
+    CONF_EXIT_AT_INTF_DOWN,
     CONF_MAX_SNIFF_BYTES,
     CONF_ENABLE_SPA_PACKET_AGING,
     CONF_MAX_SPA_PACKET_AGE,
     CONF_ENABLE_DIGEST_PERSISTENCE,
+    CONF_RULES_CHECK_THRESHOLD,
     CONF_CMD_EXEC_TIMEOUT,
     //CONF_BLACKLIST,
     CONF_ENABLE_SPA_OVER_HTTP,
@@ -297,6 +309,7 @@ enum {
     CONF_FWKNOP_RUN_DIR,
     CONF_FWKNOP_CONF_DIR,
     CONF_ACCESS_FILE,
+    CONF_ACCESS_FOLDER,
     CONF_FWKNOP_PID_FILE,
 #if USE_FILE_CACHE
     CONF_DIGEST_FILE,
@@ -305,6 +318,7 @@ enum {
 #endif
     CONF_GPG_HOME_DIR,
     CONF_GPG_EXE,
+    CONF_SUDO_EXE,
     CONF_FIREWALL_EXE,
     CONF_VERBOSE,
 #if AFL_FUZZING
@@ -366,8 +380,17 @@ typedef struct acc_stanza
     unsigned char        use_rijndael;
     int                  fw_access_timeout;
     unsigned char        enable_cmd_exec;
+    unsigned char        enable_cmd_sudo_exec;
+    char                *cmd_sudo_exec_user;
+    char                *cmd_sudo_exec_group;
+    uid_t                cmd_sudo_exec_uid;
+    gid_t                cmd_sudo_exec_gid;
     char                *cmd_exec_user;
     char                *cmd_exec_group;
+    char                *cmd_cycle_open;
+    char                *cmd_cycle_close;
+    unsigned char        cmd_cycle_do_close;
+    int                  cmd_cycle_timer;
     uid_t                cmd_exec_uid;
     gid_t                cmd_exec_gid;
     char                *require_username;
@@ -404,6 +427,16 @@ typedef struct acc_stanza
     struct acc_stanza   *next;
 } acc_stanza_t;
 
+/* A simple linked list of strings for command open/close cycles
+*/
+typedef struct cmd_cycle_list
+{
+    char                    src_ip[MAX_IPV4_STR_LEN];
+    char                   *close_cmd;
+    time_t                  expire;
+    int                     stanza_num;
+    struct cmd_cycle_list  *next;
+} cmd_cycle_list_t;
 
 /* Firewall-related data and types. */
 
@@ -585,14 +618,21 @@ typedef struct fko_srv_options
     unsigned char   fw_list;            /* List current firewall rules */
     unsigned char   fw_list_all;        /* List all current firewall rules */
     unsigned char   fw_flush;           /* Flush current firewall rules */
+    unsigned char   key_gen;            /* Generate keys and exit */
+    unsigned char   exit_after_parse_config; /* Parse config and exit */
+    unsigned char   exit_parse_digest_cache; /* Parse digest cache and exit */
+
+    /* Operational flags
+    */
     unsigned char   test;               /* Test mode flag */
     unsigned char   afl_fuzzing;        /* SPA pkts from stdin for AFL fuzzing */
     unsigned char   verbose;            /* Verbose mode flag */
-    unsigned char   exit_after_parse_config; /* Parse config and exit */
     unsigned char   enable_udp_server;  /* Enable UDP server mode */
+    unsigned char   enable_fw;          /* Command modes by themselves don't
+                                           need firewall support. */
 
     unsigned char   firewd_disable_check_support; /* Don't use firewall-cmd ... -C */
-    unsigned char   ipt_disable_check_support; /* Don't use iptables -C */
+    unsigned char   ipt_disable_check_support;    /* Don't use iptables -C */
 
     /* Flag for permitting SPA packets regardless of directionality test
      * w.r.t. the sniffing interface.  This can sometimes be useful for SPA
@@ -605,6 +645,13 @@ typedef struct fko_srv_options
     int             tcp_server_pid;
     int             lock_fd;
 
+    /* Values used in --key-gen mode only
+    */
+    char key_gen_file[MAX_PATH_LEN];
+    int  key_len;
+    int  hmac_key_len;
+    int  hmac_type;
+
 #if USE_FILE_CACHE
     struct digest_cache_list *digest_cache;   /* In-memory digest cache list */
 #endif
@@ -622,24 +669,46 @@ typedef struct fko_srv_options
     */
     char           *config[NUMBER_OF_CONFIG_ENTRIES];
 
+    /* Data elements that are derived from configuration entries - avoids
+     * calling strtol_wrapper() after the config is parsed.
+    */
+    unsigned short tcpserv_port;
+    unsigned short udpserv_port;
+    int            udpserv_select_timeout;
+    int            rules_chk_threshold;
+    int            pcap_loop_sleep;
+    int            pcap_dispatch_count;
+    int            max_sniff_bytes;
+    int            max_spa_packet_age;
+
     acc_stanza_t   *acc_stanzas;       /* List of access stanzas */
 
     /* Firewall config info.
     */
     struct fw_config *fw_config;
 
+    /* Rule checking counter - this is for garbage cleanup mode to remove
+     * any rules with an expired timer (even those that may have been
+     * added by a third-party program).
+    */
+    unsigned int check_rules_ctr;
+
+    /* Track external command execution cycles (track source IP, access.conf
+     * stanza number, and instantiation time).
+    */
+    cmd_cycle_list_t *cmd_cycle_list;
+
     /* Set to 1 when messages have to go through syslog, 0 otherwise */
     unsigned char   syslog_enable;
 
 } fko_srv_options_t;
 
-extern fko_srv_options_t options;
-
 /* For cleaning up memory before exiting
 */
 #define FW_CLEANUP          1
 #define NO_FW_CLEANUP       0
-void clean_exit(fko_srv_options_t *opts, unsigned int fw_cleanup_flag, unsigned int exit_status);
+void clean_exit(fko_srv_options_t *opts,
+        unsigned int fw_cleanup_flag, unsigned int exit_status);
 
 #endif /* FWKNOPD_COMMON_H */
 
