@@ -6,7 +6,7 @@
  * Purpose: The pcap capture routines for fwknopd.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -31,12 +31,14 @@
 
 #if USE_LIBPCAP
   #include <pcap.h>
+  #include <errno.h>
 #endif
 
 #include "fwknopd_common.h"
 #include "pcap_capture.h"
 #include "process_packet.h"
 #include "fw_util.h"
+#include "cmd_cycle.h"
 #include "log_msg.h"
 #include "fwknopd_errors.h"
 #include "sig_handler.h"
@@ -63,32 +65,13 @@ pcap_capture(fko_srv_options_t *opts)
     int                 set_direction = 1;
     int                 pcap_file_mode = 0;
     int                 status;
-    int                 useconds;
-    int                 pcap_dispatch_count;
-    int                 max_sniff_bytes;
-    int                 is_err;
+    int                 chk_rm_all = 0;
     pid_t               child_pid;
 
 #if FIREWALL_IPFW
     time_t              now;
 #endif
 
-    useconds = strtol_wrapper(opts->config[CONF_PCAP_LOOP_SLEEP],
-            0, RCHK_MAX_PCAP_LOOP_SLEEP, NO_EXIT_UPON_ERR, &is_err);
-    if(is_err != FKO_SUCCESS)
-    {
-        log_msg(LOG_ERR, "[*] invalid PCAP_LOOP_SLEEP_value");
-        clean_exit(opts, FW_CLEANUP, EXIT_FAILURE);
-    }
-
-    max_sniff_bytes = strtol_wrapper(opts->config[CONF_MAX_SNIFF_BYTES],
-            0, RCHK_MAX_SNIFF_BYTES, NO_EXIT_UPON_ERR, &is_err);
-    if(is_err != FKO_SUCCESS)
-    {
-        log_msg(LOG_ERR, "[*] invalid MAX_SNIFF_BYTES");
-        clean_exit(opts, FW_CLEANUP, EXIT_FAILURE);
-    }
-
     /* Set promiscuous mode if ENABLE_PCAP_PROMISC is set to 'Y'.
     */
     if(strncasecmp(opts->config[CONF_ENABLE_PCAP_PROMISC], "Y", 1) == 0)
@@ -117,7 +100,7 @@ pcap_capture(fko_srv_options_t *opts)
             opts->config[CONF_PCAP_INTF]);
 
         pcap = pcap_open_live(opts->config[CONF_PCAP_INTF],
-            max_sniff_bytes, promisc, 100, errstr
+            opts->max_sniff_bytes, promisc, 100, errstr
         );
 
         if(pcap == NULL)
@@ -201,14 +184,6 @@ pcap_capture(fko_srv_options_t *opts)
         clean_exit(opts, FW_CLEANUP, EXIT_FAILURE);
     }
 
-    pcap_dispatch_count = strtol_wrapper(opts->config[CONF_PCAP_DISPATCH_COUNT],
-            0, RCHK_MAX_PCAP_DISPATCH_COUNT, NO_EXIT_UPON_ERR, &is_err);
-    if(is_err != FKO_SUCCESS)
-    {
-        log_msg(LOG_ERR, "[*] invalid PCAP_DISPATCH_COUNT");
-        clean_exit(opts, FW_CLEANUP, EXIT_FAILURE);
-    }
-
     /* Initialize our signal handlers. You can check the return value for
      * the number of signals that were *not* set.  Those that were not set
      * will be listed in the log/stderr output.
@@ -257,7 +232,7 @@ pcap_capture(fko_srv_options_t *opts)
             pending_break = 1;
         }
 
-        res = pcap_dispatch(pcap, pcap_dispatch_count,
+        res = pcap_dispatch(pcap, opts->pcap_dispatch_count,
             (pcap_handler)&process_packet, (unsigned char *)opts);
 
         /* Count processed packets
@@ -288,9 +263,20 @@ pcap_capture(fko_srv_options_t *opts)
         */
         else if(res == -1)
         {
-            log_msg(LOG_ERR, "[*] Error from pcap_dispatch: %s",
-                pcap_geterr(pcap)
-            );
+            if((strncasecmp(opts->config[CONF_EXIT_AT_INTF_DOWN], "Y", 1) == 0)
+                    && errno == ENETDOWN)
+            {
+                log_msg(LOG_ERR, "[*] Fatal error from pcap_dispatch: %s",
+                    pcap_geterr(pcap)
+                );
+                clean_exit(opts, FW_CLEANUP, EXIT_FAILURE);
+            }
+            else
+            {
+                log_msg(LOG_ERR, "[*] Error from pcap_dispatch: %s",
+                    pcap_geterr(pcap)
+                );
+            }
 
             if(pcap_errcnt++ > MAX_PCAP_ERRORS_BEFORE_BAIL)
             {
@@ -309,10 +295,29 @@ pcap_capture(fko_srv_options_t *opts)
         else
             pcap_errcnt = 0;
 
-        /* Check for any expired firewall rules and deal with them.
-        */
         if(!opts->test)
-            check_firewall_rules(opts);
+        {
+            if(opts->enable_fw)
+            {
+                /* Check for any expired firewall rules and deal with them.
+                */
+                if(opts->rules_chk_threshold > 0)
+                {
+                    opts->check_rules_ctr++;
+                    if ((opts->check_rules_ctr % opts->rules_chk_threshold) == 0)
+                    {
+                        chk_rm_all = 1;
+                        opts->check_rules_ctr = 0;
+                    }
+                }
+                check_firewall_rules(opts, chk_rm_all);
+                chk_rm_all = 0;
+            }
+
+            /* See if any CMD_CYCLE_CLOSE commands need to be executed.
+            */
+            cmd_cycle_close(opts);
+        }
 
 #if FIREWALL_IPFW
         /* Purge expired rules that no longer have any corresponding
@@ -329,7 +334,7 @@ pcap_capture(fko_srv_options_t *opts)
         }
 #endif
 
-        usleep(useconds);
+        usleep(opts->pcap_loop_sleep);
     }
 
     pcap_close(pcap);
