@@ -6,7 +6,7 @@
  * Purpose: Header file for process_packet and other fwknopd code.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -33,7 +33,6 @@
 
 #define IPV4_VER_MASK   0x15
 #define MIN_IPV4_WORDS  0x05
-#define FCS_HEADER_LEN  0x04
 
 /* For items not defined by this system
 */
@@ -47,7 +46,8 @@
 /* Prototypes
 */
 #if USE_LIBPCAP
-void process_packet(unsigned char *args, const struct pcap_pkthdr *packet_header, const unsigned char *packet);
+void process_packet(unsigned char *args,
+        const struct pcap_pkthdr *packet_header, const unsigned char *packet);
 #endif
 
 #endif  /* PROCESS_PACKET_H */
