@@ -4,7 +4,7 @@
  * @brief   General/Generic functions for the fwknop server.
  *
  *  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
- *  Copyright (C) 2009-2014 fwknop developers and contributors. For a full
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
  *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
@@ -31,76 +31,82 @@
 #include "replay_cache.h"
 #include "config_init.h"
 #include "fw_util.h"
+#include "cmd_cycle.h"
 
-#include <stdarg.h>
-
-#define ASCII_LEN 16
-
-/* Generic hex dump function.
+/* Basic directory/binary checks (stat() and whether the path is actually
+ * a directory or an executable).
 */
-void
-hex_dump(const unsigned char *data, const int size)
+static int
+is_valid_path(const char *path, const int file_type)
 {
-    int ln=0, i=0, j=0;
-    char ascii_str[ASCII_LEN+1] = {0};
-
-    for(i=0; i<size; i++)
+    if(strnlen(path, MAX_PATH_LEN) == MAX_PATH_LEN)
     {
-        if((i % ASCII_LEN) == 0)
-        {
-            printf(" %s\n  0x%.4x:  ", ascii_str, i);
-            memset(ascii_str, 0x0, ASCII_LEN-1);
-            j = 0;
-        }
-
-        printf("%.2x ", data[i]);
-
-        ascii_str[j++] = (data[i] < 0x20 || data[i] > 0x7e) ? '.' : data[i];
-
-        if(j == 8)
-            printf(" ");
+        log_msg(LOG_ERR, "[-] Provided path is too long");
+        return(0);
     }
 
-    /* Remainder...
-    */
-    ln = strlen(ascii_str);
-    if(ln > 0)
-    {
-        for(i=0; i < ASCII_LEN-ln; i++)
-            printf("   ");
-        if(ln < 8)
-            printf(" ");
-
-        printf(" %s\n\n", ascii_str);
-    }
-    return;
-}
-
-/* Basic directory checks (stat() and whether the path is actually
- * a directory).
-*/
-int
-is_valid_dir(const char *path)
-{
-#if HAVE_STAT
+#if HAVE_STAT || HAVE_LSTAT
     struct stat st;
 
-    /* If we are unable to stat the given dir, then return with error.
+    /* If we are unable to stat the given path, then return with error.
     */
-    if(stat(path, &st) != 0)
+  #if HAVE_LSTAT /* prefer lstat() to stat() */
+    if(lstat(path, &st) != 0)
     {
-        log_msg(LOG_ERR, "[-] unable to stat() directory: %s: %s",
+        log_msg(LOG_ERR, "[-] unable to lstat() path: %s: %s",
             path, strerror(errno));
         return(0);
     }
-
-    if(!S_ISDIR(st.st_mode))
+  #else
+    if(stat(path, &st) != 0)
+    {
+        log_msg(LOG_ERR, "[-] unable to stat() path: %s: %s",
+            path, strerror(errno));
         return(0);
-#endif /* HAVE_STAT */
+    }
+  #endif
+
+    if(file_type == IS_DIR)
+    {
+        if(!S_ISDIR(st.st_mode))
+            return(0);
+    }
+    else if(file_type == IS_EXE)
+    {
+        if(!S_ISREG(st.st_mode) || ! (st.st_mode & S_IXUSR))
+            return(0);
+    }
+    else if(file_type == IS_FILE)
+    {
+        if(!S_ISREG(st.st_mode))
+            return(0);
+    }
+    else
+        return(0);
+
+#endif /* HAVE_STAT || HAVE_LSTAT */
 
     return(1);
 }
 
+int
+is_valid_dir(const char *path)
+{
+    return is_valid_path(path, IS_DIR);
+}
+
+int
+is_valid_exe(const char *path)
+{
+    return is_valid_path(path, IS_EXE);
+}
+
+int
+is_valid_file(const char *path)
+{
+    return is_valid_path(path, IS_FILE);
+}
+
 int
 verify_file_perms_ownership(const char *file)
 {
@@ -163,99 +169,56 @@ verify_file_perms_ownership(const char *file)
     return 1;
 }
 
-static int
-add_argv(char **argv_new, int *argc_new,
-        const char *new_arg, const fko_srv_options_t * const opts)
+void
+truncate_partial_line(char *str)
 {
-    int buf_size = 0;
+    int i, have_newline=0;
 
-    if(opts->verbose > 3)
-        log_msg(LOG_INFO, "[+] add_argv() + arg: %s", new_arg);
-
-    buf_size = strlen(new_arg) + 1;
-    argv_new[*argc_new] = calloc(1, buf_size);
-
-    if(argv_new[*argc_new] == NULL)
+    if(str != NULL && str[0] != 0x0)
     {
-        log_msg(LOG_INFO, "[*] Memory allocation error.");
-        return 0;
-    }
-    strlcpy(argv_new[*argc_new], new_arg, buf_size);
-
-    *argc_new += 1;
-
-    if(*argc_new >= MAX_CMDLINE_ARGS-1)
-    {
-        log_msg(LOG_ERR, "[*] max command line args exceeded.");
-        return 0;
-    }
-
-    argv_new[*argc_new] = NULL;
-
-    return 1;
-}
-
-int
-strtoargv(const char * const args_str, char **argv_new, int *argc_new,
-        const fko_srv_options_t * const opts)
-{
-    int       current_arg_ctr = 0, i;
-    char      arg_tmp[MAX_LINE_LEN] = {0};
-
-    for (i=0; i < (int)strlen(args_str); i++)
-    {
-        if (!isspace(args_str[i]))
+        for (i=0; i < strlen(str); i++)
         {
-            arg_tmp[current_arg_ctr] = args_str[i];
-            current_arg_ctr++;
-        }
-        else
-        {
-            if(current_arg_ctr > 0)
+            if(str[i] == 0x0a)
             {
-                arg_tmp[current_arg_ctr] = '\0';
-                if (add_argv(argv_new, argc_new, arg_tmp, opts) != 1)
-                {
-                    free_argv(argv_new, argc_new);
-                    return 0;
-                }
-                current_arg_ctr = 0;
+                have_newline = 1;
+                break;
+            }
+        }
+
+        /* Don't zero out any data unless there is at least
+         * one newline
+        */
+        if(have_newline)
+        {
+            for (i=strlen(str)-1; i > 0; i--)
+            {
+                if(str[i] == 0x0a)
+                    break;
+                str[i] = 0x0;
             }
         }
     }
+    return;
+}
 
-    /* pick up the last argument in the string
-    */
-    if(current_arg_ctr > 0)
+/* Simple test to see if a string only contains digits
+*/
+int
+is_digits(const char * const str)
+{
+    int i;
+    if (str != NULL && str[0] != 0x0)
     {
-        arg_tmp[current_arg_ctr] = '\0';
-        if (add_argv(argv_new, argc_new, arg_tmp, opts) != 1)
+        for (i=0; i<strlen(str); i++)
         {
-            free_argv(argv_new, argc_new);
-            return 0;
+            if(!isdigit(str[i]))
+                return 0;
+            i++;
         }
     }
     return 1;
 }
 
-void
-free_argv(char **argv_new, int *argc_new)
-{
-    int i;
-
-    if(argv_new == NULL || *argv_new == NULL)
-        return;
-
-    for (i=0; i < *argc_new; i++)
-    {
-        if(argv_new[i] == NULL)
-            break;
-        else
-            free(argv_new[i]);
-    }
-    return;
-}
-
 void
 clean_exit(fko_srv_options_t *opts, unsigned int fw_cleanup_flag, unsigned int exit_status)
 {
@@ -266,7 +229,7 @@ clean_exit(fko_srv_options_t *opts, unsigned int fw_cleanup_flag, unsigned int e
     }
 #endif
 
-    if(!opts->test && (fw_cleanup_flag == FW_CLEANUP))
+    if(!opts->test && opts->enable_fw && (fw_cleanup_flag == FW_CLEANUP))
         fw_cleanup(opts);
 
 #if USE_FILE_CACHE
@@ -274,6 +237,7 @@ clean_exit(fko_srv_options_t *opts, unsigned int fw_cleanup_flag, unsigned int e
 #endif
 
     free_logging();
+    free_cmd_cycle_list(opts);
     free_configs(opts);
     exit(exit_status);
 }
