@@ -421,33 +421,6 @@
         'key_file' => $cf{'rc_named_key'},
     },
 
-    ### --key-gen tests
-    {
-        'category' => 'Rijndael',
-        'subcategory' => 'client',
-        'detail'   => '--key-gen',
-        'function' => \&generic_exec,
-        'cmdline'  => "$fwknopCmd --key-gen",
-        'positive_output_matches' => [qr/^KEY_BASE64\:?\s\S{10}/,
-            qw/HMAC_KEY_BASE64\:?\s\S{10}/],
-    },
-    {
-        'category' => 'Rijndael',
-        'subcategory' => 'client',
-        'detail'   => "--key-gen $uniq_keys key uniqueness",
-        'function' => \&key_gen_uniqueness,
-        'cmdline'  => "$fwknopCmd --key-gen",   ### no valgrind string (too slow for 100 client exec's)
-        'disable_valgrind' => $YES,
-    },
-    {
-        'category' => 'Rijndael',
-        'subcategory' => 'client',
-        'detail'   => '--key-gen to file',
-        'function' => \&generic_exec,
-        'cmdline'  => "$fwknopCmd --key-gen --key-gen-file $key_gen_file",
-        'positive_output_matches' => [qr/Wrote.*\skeys/],
-    },
-
     ### rc file tests
     {
         'category' => 'Rijndael',
